@@ -6,7 +6,7 @@ JSON::Validator - Validate data against a JSON schema
 
 =head1 VERSION
 
-0.48
+0.49
 
 =head1 SYNOPSIS
 
@@ -92,7 +92,7 @@ use constant DEBUG => $ENV{JSON_VALIDATOR_DEBUG} || $ENV{SWAGGER2_DEBUG} || 0;
 use constant WARN_ON_MISSING_FORMAT => $ENV{JSON_VALIDATOR_WARN_ON_MISSING_FORMAT}
   || $ENV{SWAGGER2_WARN_ON_MISSING_FORMAT} ? 1 : 0;
 
-our $VERSION = '0.48';
+our $VERSION = '0.49';
 
 sub E { bless {path => $_[0] || '/', message => $_[1]}, 'JSON::Validator::Error'; }
 sub S { Mojo::Util::md5_sum(Data::Dumper->new([@_])->Sortkeys(1)->Useqq(1)->Dump); }
@@ -101,6 +101,7 @@ sub S { Mojo::Util::md5_sum(Data::Dumper->new([@_])->Sortkeys(1)->Useqq(1)->Dump
 
 =head2 coerce
 
+  $self = $self->coerce(1);
   $bool = $self->coerce;
 
 Set this to true if you want to coerce numbers into string and the other way around.
@@ -312,46 +313,57 @@ sub _coerce_by_collection_format {
 sub _load_schema {
   my ($self, $url) = @_;
   my ($namespace, $scheme) = ("$url", "file");
-
-  if ($namespace =~ m!^data?://(.*)!) {
-    my ($module, $section) = split '/', $1, 2;
-    require Mojo::Loader;
-    return $self->_register_document(Mojo::Loader::data_section($module, $section), $namespace);
-  }
+  my $doc;
 
   if ($namespace =~ m!^https?://!) {
     $url = Mojo::URL->new($url);
     ($namespace, $scheme) = ($url->clone->fragment(undef)->port(undef)->to_string, $url->scheme);
   }
-
-  return $self->{cached}{$namespace} if $self->{cached}{$namespace};
-
-  warn "[JSON::Validator] Loading schema from $url ($namespace)\n" if DEBUG;
-  return $self->_register_document(Mojo::Util::slurp($url), $namespace) if $scheme eq 'file';
-
-  my $cache = File::Spec->catfile($self->_cache_dir, Mojo::Util::md5_sum($namespace));
-  return $self->_register_document(Mojo::Util::slurp($cache), $namespace) if -r $cache;
-
-  my $tx  = $self->ua->get($url);
-  my $doc = $tx->res->body;
-
-  if ($self->_cache_dir and -w $self->_cache_dir) {
-    Mojo::Util::spurt($doc, File::Spec->catfile($self->_cache_dir, Mojo::Util::md5_sum($url)));
+  elsif ($namespace =~ m!^data://(.*)!) {
+    $scheme = 'data';
   }
 
-  return $self->_register_document($doc, $namespace);
+  # Make sure we create the correct namespace if not already done by Mojo::URL
+  $namespace =~ s!#.*$!! if $namespace eq $url;
+
+  return $self->{cached}{$namespace} if $self->{cached}{$namespace};
+  return eval {
+    warn "[JSON::Validator] Loading schema from $url ($namespace)\n" if DEBUG;
+    $doc
+      = $scheme eq 'file' ? Mojo::Util::slurp($url)
+      : $scheme eq 'data' ? $self->_load_schema_from_data($url, $namespace)
+      :                     $self->_load_schema_from_url($url, $namespace);
+    $self->_register_document($self->_load_schema_from_text($doc));
+  } || do {
+    die "Could not load document from $url: $@ ($doc)" if DEBUG;
+    die "Could not load document from $url: $@";
+  };
+}
+
+sub _load_schema_from_data {
+  my ($self, $url, $namespace) = @_;
+  require Mojo::Loader;
+  $namespace =~ m!^data://([^/]+)/(.*)$!;
+  Mojo::Loader::data_section($1 || 'main', $2 || $namespace);
+}
+
+sub _load_schema_from_text {
+  $_[1] =~ /^\s*\{/s ? Mojo::JSON::decode_json($_[1]) : _load_yaml($_[1]);
+}
+
+sub _load_schema_from_url {
+  my ($self, $url, $namespace) = @_;
+  my $cache_file = File::Spec->catfile($self->_cache_dir, Mojo::Util::md5_sum($namespace));
+
+  return Mojo::Util::slurp($cache_file) if -r $cache_file;
+  my $doc = $self->ua->get($url)->res->body;
+  Mojo::Util::spurt($doc, $cache_file) if $self->_cache_dir and -w $self->_cache_dir;
+  return $doc;
 }
 
 sub _register_document {
   my ($self, $doc, $namespace) = @_;
 
-  unless (ref $doc) {
-    unless (eval { $doc = $doc =~ /^\s*\{/s ? Mojo::JSON::decode_json($doc) : _load_yaml($doc) }) {
-      die "Could not load document from $namespace: $@ ($doc)" if DEBUG;
-      die "Could not load document from $namespace: $@";
-    }
-  }
-
   $doc = Mojo::JSON::Pointer->new($doc);
   $namespace = Mojo::URL->new($namespace) unless ref $namespace;
   $namespace->fragment(undef)->port(undef);
